;Mini-Shell by chicane
;version .62
;more optimization from version .6
#include "asm86.h"
#include "ti86asm.inc"
#include "ti86ops.inc"

_MOV10TOOP1     		equ     $42D7	;Move 10bytes a hl to OP1
_MOV10TOOP2     		equ     $42DF	;Move 10bytes a hl to OP1
_cpOP1OP2               equ     $41fb   ;compare OP1 to OP2
_homeup         		equ     $4A95	;resets _curRow etc.

.org _asm_exec_ram

;<--------- Identification---------->
	nop
	jp  ASMstart
	.dw $0000
	.dw PrgmName

ASMstart:
	call    BUSY_OFF			;turn busy indicator off
	call    _clrScrn			;clear the screen
	call    clrPlot				;cleat the _plotSScrn
	call    Display
	call	DispBox

FirstPrgm:
	ld		hl,AbsFirst			;Load address of the first posible program name into hl
	call	_MOV10TOOP1			;move it to OP1
	xor		a					;clear a
 	call	$514B				;Find Next Program Variable
	xor		a					;clear a
	call	$514B				;do it again
	jp		c,NoPrgms			;If 2 programs are not found (minishel + 1 other) then exit w/msg
 	ld		hl,AbsFirst			;reset hl to first possible prog
	call	_MOV10TOOP1			;move to op1
	call	NextPrgm			;Finds the next program that is not minishel
	jp		GetKey1				;main getkey loop

Display:
	ld      hl,$0001			;duh
	ld      (_penCol), hl		;set up where to put text
	ld      hl,PrgmName			;load the text address
	call    _vputs				;output in variable width text format.
	
	ld      a,$58			
	ld      (_penCol),a
	ld      hl,Version			;same as above
	call    _vputs
	
	ld      hl,$3A19
	ld      (_penCol),hl		;blah blah
	ld      hl,EMail
	call    _vputs
	
	ld      hl,$FFA0			
	call    Invert				;Draws a revers video bar 
	ld      hl,$FC00	
	call    Invert
	ret

DispBox:
	set		3,(iy+5)			;set reverse video mode
	ld		hl,$0302			;output the "Programs List" text
	ld		(_curRow),hl
	ld		hl,PrgmsMenu
	call	_puts
	res		3,(iy+5)			;reset reverse video mode
	ret

GetKey1:						;Key Check Routine
	HALT						;saves battery power
	call    GET_KEY				;Gets last key pressed
	cp      K_NOKEY				;compare to nokey or 0
	jr      z,GetKey1			;jump back up
	cp      K_EXIT				;et cetera 
	jp      z,Exit
	cp      K_UP
	call    z,LastPrgm
	cp      K_DOWN
	call    z,NextPrgm
	cp      K_PLUS
	jp      z,ContUp
	cp      K_MINUS
	jp      z,ContDown
	cp      K_CLEAR
	jp      z,ShutOff
	cp		K_ENTER
	jp		z,Exec
	cp		K_DEL
	jp		z,Delete
	cp		K_SECOND
	jp		z,Exec
	jp      GetKey1

;<-------Utility Routines---------->
ContUp:       					;changes the contrast up
	ld      a,($C008)			;contrast value
	inc     a					;incriment it
	ld      ($C008),a			;load it back in
	out     (2),a				;and out to the screen port
	xor	a						;clear a
	jp      GetKey1				;back to main loop

ContDown:						;changes contrast down
	ld      a,($C008)			;contrast value
	dec     a					;decriment it
	ld      ($C008),a			;load it back in
	out     (2),a				;and to the screen port also
	xor	a						;clear a
	jp      GetKey1				;jump to main loop

clrPlot:						;clears the _plotSScrn
	ld      hl,$C9FA			;this routine written by Dark Ryder
	ld      de,$C9FB			;no need to explain
	ld      bc,$3FF				;
	ld      a,(hl)
	ldir
	ret
	
Invert:							;Inverts a 7 line place on the screen.
	ld      bc,112				;written by Andreas Ess
InvertLoop:
	ld      a,(hl)
	cpl
	ld      (hl),a
	inc     hl
	dec     bc
	ld      a,b
	or      c
	jr      nz,InvertLoop
	ret

LastPrgm:						;Finds the Previous program in the VAT
	xor		a					;clear a
	call	$514F				;the actuall call
	xor		a					;clear it again
	ret		c					;return if no more found
	ld		hl,MiniShell		;load the VAT Entry of Mini-Shell into hl
	call	_MOV10TOOP2			;move it to op2
	call	_cpOP1OP2			;compare Mini-Shell to op1
	jp		z,FoundMini1		;Jump to FoundMini1 if they are the same
	jp		DisplayP			;else Display the program

FoundMini1:						;skips minishell
	xor		a					;clear a
	call	$514F				;check for program before minishell
	jp		nc,DisplayP			;diplay if one exists 
	xor		a
	call	$514B				;then find the program before minishel
	jp		DisplayP			;then display it

NextPrgm:						;Finds the next Program
	xor		a					;clear a
 	call	$514B				;opposite of $514F
	xor		a
	ret	    c					;same as above
	ld		hl,MiniShell
	call	_MOV10TOOP2
	call	_cpOP1OP2
	jp		z,FoundMini2
	jp  	DisplayP			;blah, blah

FoundMini2:
	xor		a
	call	$514B
	jp		nc,DisplayP
	xor		a
	call	$514F
	jp		DisplayP

DisplayP:						;Display Routine
	ld		hl,$0603			;place to start text
	ld		(_curRow),hl		;load it into _penCol
	ld		hl,_OP1+2			;what to display
	call	_puts				;output to screen
	ld		hl,ClearLine		;add spaces to end to cover up previous program if was a longer name
	call	_puts				;output
	xor		a					;clear a
	ret

Exec:
	rst		10h					;find the address of the program in OP1
	call	$46AB				;check for asm of basic
	jr		nz,BasicExec		;if basic, jump down
	call	_exec_assembly		;else, exec it.
	xor		a					;clear a
	jp		ASMstart			;restart

BasicExec:
	call	BUSY_ON				;turn busy indicator on
	call	$4C57				;exececute basic program
	jp		ASMstart			;restart

Delete:							;Delete current program
	rst		10h					;find address
	call	_delvar				;delete it
	xor		a					;clear a
	jp		ASMstart			;restart

ShutOff:						;shut off routine
	ld      a,1					
	out     ($03),a             ;output low power state to port 3                   
	ld      a, $0b
	HALT						;wait til something happens
	out     ($03),a				;turn the calc back on
	res     4,(IY+$09)			;tell the calc that on was not pressed
	jp      ASMstart			;Restart

NoPrgms:						;Tells you that you dont have any programs except minishel
	call	_clrScrn			;clear somthing
	call	_homeup				;reset _curRow etc
	ld		hl,NoProgs			;load the string into hl
	call	_puts				;output it
	ret							;exit to ti-os
	
Exit:							;Exit minishell
	call	_clrScrn			;clear it again
	ret							;return to ti-os

;<------------ Strings used in program-------------->

NoProgs:
	.db "No Programs Detected!",0

PrgmsMenu:
	.db " Program  List ",0

AbsFirst:
	.db $12,$01,$00,$00,$00,$00,$00,$00,$00,$00

MiniShell:
	.db $12,$08,"minishel",$00,0

ClearLine:
	.db $00,$00,$00,$00,$00,$00,$00,0

EMail:  
	.db "chicane@reninet.com",0

PrgmName:
	.db "Mini-Shell",0

Version:
	.db "version .62",0
.end

